@@ -632,9 +632,25 @@ static int check_prefixes(char **prefixes, int num_prefixes, const char *path)
 		return 0;
 
 	size_t i;
-	for (i = 0; i < num_prefixes; ++i)
-		if (prefixes[i] && !strncmp(path, prefixes[i], strlen(prefixes[i])))
-			return 1;
+	for (i = 0; i < num_prefixes; ++i) {
+		if (unlikely(!prefixes[i]))
+			continue;
+
+		size_t prefix_len = strlen(prefixes[i]);
+		/* Start with a regular prefix match for speed */
+		if (strncmp(path, prefixes[i], prefix_len))
+			continue;
+
+		/* Now, if prefix did not end with a slash, we need to make sure
+		 * we are not matching in the middle of a filename. So check
+		 * whether the match is followed by a slash, or NUL.
+		 */
+		if (prefixes[i][prefix_len-1] != '/'
+				&& path[prefix_len] != '/' && path[prefix_len] != '\0')
+			continue;
+
+		return 1;
+	}
 
 	return 0;
 }
@@ -650,8 +666,10 @@ static bool symlink_func(int sb_nr, int flags, const char *abs_path)
 	      sb_nr == SB_NR_LCHOWN   ||
 	      sb_nr == SB_NR_REMOVE   ||
 	      sb_nr == SB_NR_RENAME   ||
+	      sb_nr == SB_NR_RENAMEAT ||
 	      sb_nr == SB_NR_RMDIR    ||
-	      sb_nr == SB_NR_SYMLINK))
+	      sb_nr == SB_NR_SYMLINK  ||
+	      sb_nr == SB_NR_SYMLINKAT))
 	{
 		/* These funcs sometimes operate on symlinks */
 		if (!((sb_nr == SB_NR_FCHOWNAT ||
@@ -710,6 +728,7 @@ static int check_access(sbcontext_t *sbcontext, int sb_nr, const char *func,
 	     sb_nr == SB_NR_EXECV     ||
 	     sb_nr == SB_NR_EXECVP    ||
 	     sb_nr == SB_NR_EXECVE    ||
+	     sb_nr == SB_NR_EXECVPE   ||
 	     sb_nr == SB_NR_FEXECVE))
 	{
 		retval = check_prefixes(sbcontext->read_prefixes,
@@ -791,46 +810,6 @@ static int check_access(sbcontext_t *sbcontext, int sb_nr, const char *func,
 			goto out;
 		}
 
-		/* XXX: Hack to enable us to remove symlinks pointing to
-		 * protected stuff.  First we make sure that the passed path
-		 * is writable, and if so, check if it's a symlink, and give
-		 * access only if the resolved path of the symlink's parent
-		 * also have write access.  We also want to let through funcs
-		 * whose flags say they will operate on symlinks themselves
-		 * rather than dereferencing them.
-		 */
-		if (sym_func) {
-			/* Check if the symlink unresolved path have access */
-			retval = check_prefixes(sbcontext->write_prefixes,
-						sbcontext->num_write_prefixes, abs_path);
-			if (1 == retval) { /* Does have write access on path */
-				char *dname, *dname_buf, *rpath;
-
-				dname_buf = xstrdup(abs_path);
-				dname = dirname(dname_buf);
-				/* Get symlink resolved path */
-				rpath = resolve_path(dname, 1);
-				free(dname_buf);
-				if (NULL == rpath)
-					/* Don't really worry here about
-					 * memory issues */
-					goto unlink_hack_end;
-
-				/* Now check if the symlink resolved path have access */
-				retval = check_prefixes(sbcontext->write_prefixes,
-							sbcontext->num_write_prefixes,
-							rpath);
-				free(rpath);
-				if (1 == retval) {
-					/* Does have write access on path, so
-					 * enable the hack as it is a symlink */
-					result = 1;
-					goto out;
-				}
-			}
-		}
- unlink_hack_end: ;
-
 		/* Hack to allow writing to '/proc/self/fd' #91516.  It needs
 		 * to be here as for each process, the '/proc/self' symlink
 		 * will differ ...
@@ -1222,7 +1201,7 @@ char **sb_check_envp(char **envp, size_t *mod_cnt, bool insert)
 		if (mod_cnt) {
 			str_list_for_each_item(envp, entry, count) {
 				for (i = 0; i < num_vars; ++i)
-					if (is_env_var(entry, vars[i].name, vars[i].len)) {
+					if (i != 12 && is_env_var(entry, vars[i].name, vars[i].len)) {
 						(*mod_cnt)++;
 						goto skip;
 					}
@@ -1231,7 +1210,7 @@ char **sb_check_envp(char **envp, size_t *mod_cnt, bool insert)
 			}
 		} else {
 			for (i = 0; i < num_vars; ++i)
-				unsetenv(vars[i].name);
+				if (i != 12) unsetenv(vars[i].name);
 		}
 	} else {
 		if (mod_cnt) {
